classdef EPSSP

   properties
   
      model = "EPSSP"
      version = "0.9.0"
      
   end
   
   methods(Static)
   
      %%%%%%%%%%%%%%%%%%%%%%%
      % Basic communication %
      %%%%%%%%%%%%%%%%%%%%%%%

      function init(ARG)
         % EPSSP.init('192.168.1.100');    opens LAN device
         % EPSSP.init(1);                  opens last USB device
         % EPSSP.init(0);                  shows list of USB devices and lets user chose one
         % EPSSP.init;                     same as eps=EPSSP(1);

         if (nargin<1)
            ARG=1; % connect to last device
         end;
         
         global EPSSPHandle
         if length(ARG)>6&&sum(class(ARG)=='char')==4  % LAN DEVICE
            EPSSPHandle=NOVTCP(ARG);
         else % USB DEVICE
            EPSSPHandle=NOVUSB('LastDevEPSSP.mat', ARG);
         end;
      end;
      
      function close
         global EPSSPHandle
         EPSSPHandle.close
      end;
      
      function [res ok] = read(addr)
         
         global EPSSPHandle
         
         [res ok] = EPSSPHandle.read(addr);
         
         if ok==0, % su success in reading. Try to connect again
            EPSSP.init;
            [res ok] = EPSSPHandle.read(addr);
         end;
         
      end;
      

      function ok = write(addr, data)
      
         global EPSSPHandle
         ok = EPSSPHandle.write(addr, data);
         if ok==0, % su success in writing. Try to connect again
            EPSSP.init;
            [res ok] = EPSSPHandle.write(addr, data);
         end;
         
         % pause to force gap between write commands
         pausetime = 0.0010; % pause 1 ms at FT232R
         mstic=tic;
         mspause=0;
         while mspause<pausetime,
            mspause=toc(mstic);
         end;
         
         
      end;
      
    
      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%
      % General instrument data %
      %%%%%%%%%%%%%%%%%%%%%%%%%%%
      
      function r = getfirmware % as string
         r = sprintf('%4X', EPSSP.read(84));
      end
      
      
      %function r = getserialnumber
      %   r = EPSSP.read(91);
      %end;
      
      %function r = getmoduletype % as string
      %   str=[];
      %   for ii=0:15,
      %      dummy = EPSSP.read(96+ii);
      %      str=[ str char(bitshift(dummy, -8)) ];
      %      str=[ str mod(dummy, 2^8) ];
      %   end;
      %   r = str;
      %end
      
      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      % Device specific functions %
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

      function r = gettemperature
         r = EPSSP.read(60)/16;
      end;
      
      function setval(value) % as integer 0..2^12-1
         value = max(0, min(2^12-1, round(value)))
         EPSSP.write(8, value);
      end;
      
      function r = getval
         r = EPSSP.read(8);
      end;

      
    
      function shutdown(sdn) % 1: shutdown dc-converter
         sdn = max(0, min(1, sdn))
         EPSSP.write(9, sdn);
      end;
      
      function r= getshutdown
         r = EPSSP.read(9);
      end;
      
      
      
      function setnumstates(num) % number of states: 3 or 4
         num = max(3, min(4, num))
         EPSSP.write(10, num);
      end;

      function r = getnumstates 
         r = EPSSP.read(10);
      end;

      
      
      
      
      

   end;

end