%classdef NOVTCP < matlab.System
classdef NOVTCP < handle

   properties
   
      version = "0.9.0"
      IPADDR
      Handle
      isOctave


   end


   methods
   
      function obj = NOVTCP(IPADDR)
      
         obj.isOctave = exist('OCTAVE_VERSION', 'builtin') ~= 0;
      
         %disp(sprintf('IPADDR = %s;', IPADDR));
         obj.IPADDR = IPADDR;
         init(obj);
         
      end;
      
      function s = getip(obj)
         s = obj.IPADDR;
      end;

      function init(obj)
      
         if obj.isOctave,
         
            %pkg load instrument-control
            obj.Handle = tcpclient(obj.IPADDR, 5025);
            
            obj.Handle
            
            TCPHandle = obj.Handle
            
            BytesAvailable = TCPHandle.NumBytesAvailable
            
         else
            
            obj.Handle = tcpip(obj.IPADDR, 5025); 

            % Set size of receiving buffer, if needed. 
            set(obj.Handle, 'InputBufferSize', 32768); % 2 cycles * 16kB is the W5500 TX buffer size
            obj.Handle.transferDelay='off'; %  If TransferDelay is off, data is sent immediately to the network.

            fopen(obj.Handle);

         end;

      end;
      
      function close(obj)
         if obj.isOctave,
            clear obj.Handle;
         else
            fclose(obj.Handle); 
         end;
      end;
      
      function ok = write(obj, addr, data)
      
         TCPHandle = obj.Handle;
      
         seq1 = [  87 bitshift(addr, -8) mod(addr, 2^8) bitshift(data, -8) mod(data, 2^8)];

         if obj.isOctave,
            write(TCPHandle, char(seq1));
         else
            fwrite(TCPHandle,seq1);
         end;
         
         ok = 1;


      end;
      
      function [res ok] = read(obj, addr)
      
         %pkg load instrument-control
      
      
         TCPHandle = obj.Handle;
      
         debugmode=0;

         ok=0;

         if length(addr)==1,
            seq1 = [ 82 bitshift(addr, -8) mod(addr, 2^8)];
         else
            seq1 = [ 77 length(addr) ];
            for ii=1:length(addr),
               seq1 = [ seq1 bitshift(addr(ii), -8) mod(addr(ii), 2^8)];
            end;
         end;


         % erase pending data
         if obj.isOctave,
            BytesAvailable = TCPHandle.NumBytesAvailable;
         else
            BytesAvailable = TCPHandle.BytesAvailable;
         end;
         
         if BytesAvailable,
            if obj.isOctave,
               A = read(TCPHandle, NumBytesAvailable);
            else
               A = fread(TCPHandle, BytesAvailable);
            end;
         end;

         tic
         if obj.isOctave,
            write(TCPHandle, char(seq1));
         else
            fwrite(TCPHandle,seq1);
         end;

         BytesAvailable=0;
         StopCounter=0;
         while BytesAvailable<2*length(addr) & StopCounter<2000,
            if obj.isOctave,
               BytesAvailable = TCPHandle.NumBytesAvailable;
            else
               BytesAvailable = TCPHandle.BytesAvailable;
            end;
            StopCounter = StopCounter+1;
            pause(.001);
         end;

         if debugmode
            toc % 10..200 ms with PM1000
         end;


         res=zeros(length(addr), 1);
         if BytesAvailable>=2*length(addr),
            if obj.isOctave,
               A = read(TCPHandle, NumBytesAvailable);
            else
               A = fread(TCPHandle, BytesAvailable);
            end;
            for ii=1:length(addr),
               res(ii) = A(2*(ii-1)+1)*2^8 + A(2*(ii-1)+2);
               ok=1;
            end;
         else
            BytesAvailable
            StopCounter
         end;

         if debugmode
            BytesAvailable
         end
         
      end;
      
      
      
      % only for PM1000: Reads one block of the Poincaré sphere frame buffer
      function [res ok] = readfr(obj, BlockNr)
      
         TCPHandle = obj.Handle;
      
         NumBytes = 2^12; % one complete frame

         debugmode=0;

         ok=0;

         tic

         % request data stream
         write(obj, 512 + 105, BlockNr);
         pause(.001);
         write(obj, 512 + 104, 27273);
         pause(.001);


         BytesAvailable=0;
         StopCounter=0;
         while BytesAvailable<NumBytes & StopCounter<2000,
            BytesAvailable = TCPHandle.BytesAvailable;
            StopCounter = StopCounter+1;
            pause(.001);
         end;

         if debugmode
            toc % 10..200 ms with PM1000
         end;


         res=0;
         if BytesAvailable>=NumBytes,
            res = fread(TCPHandle, BytesAvailable);
            ok=1;
         else
            BytesAvailable
            StopCounter
         end;

         if debugmode
            BytesAvailable
         end
      end;
      
      
      % only for PM1000: Reads data from SDRAM
      function [res, ok] = readhspm(obj, startaddr, numaddr)
      
         TCPHandle = obj.Handle;
      
         %buffersizebytes=2^12;
         buffersizebytes=2^14;



         %cycles=2; %equals numhspack in vhdl
         %cycles=4; %equals numhspack in vhdl
         %cycles=8; %equals numhspack in vhdl
         %cycles=16; %equals numhspack in vhdl
         %cycles=32; %equals numhspack in vhdl

         %cycles=max(1, min(32, numaddr*8/buffersizebytes))
         %cycles=max(1, min(32, numaddr*8/buffersizebytes)) % debug
         cycles=max(1, min(32, ceil(numaddr*8/buffersizebytes))); % debug
         cycles=2^ceil(log2(cycles));
         
         numaddr=ceil(numaddr/cycles)*cycles;


         buffersizeaddr=buffersizebytes/8*cycles;



            
         packets_transferred=0;



         res=[];

         tic;
            
         while packets_transferred<numaddr,
            
             packetsinthissequence=min(buffersizeaddr, numaddr-packets_transferred);
            
             startaddrseq = startaddr + packets_transferred;

             [res_seq, ok, timeoutreached] = PM1000_requestpackets(obj, startaddrseq, packetsinthissequence, cycles);
             
             if timeoutreached,
               break;
             end;
            
             if ok,
                 packets_transferred=packets_transferred+packetsinthissequence;
                 res=[res res_seq];
             else
                 disp('HS data transmission failed. Trying again...')
             end;
             
             
                 

         end;

         %toc
            
         %pause(.1); BytesAvailable = TCPHandle.BytesAvailable   

      end
      
      
      
      % only for PM1000: Requests one or more packets of data from SDRAM
      function [res, ok, timeoutreached] = PM1000_requestpackets(obj, startaddrseq, packetsinthissequence, cycles)

         TCPHandle = obj.Handle;
         

         % reqisters for HS-configuration
         addr_data = [ 512+105 mod(startaddrseq, 2^16);
                       512+106 bitshift(startaddrseq, -16) + log2(cycles)*2^12;
                       %512+107 packetsinthissequence;
                       512+107 round(packetsinthissequence/cycles);
                       512+104 39294]; % trigger code for HS-transfer over LAN
         for ii=1:4,
              seq(:,ii) = [87 bitshift(addr_data(ii,1), -8) mod(addr_data(ii,1), 2^8) bitshift(addr_data(ii,2), -8) mod(addr_data(ii,2), 2^8) ];
         end;
         
         %writeepstcp(512+105,0); % set address: lower 16 bits
         
         % send the request
         fwrite(TCPHandle, reshape(seq, 1, 20)); % writing as one packet improves timing compared to single write packets
         
         timeouttic=tic;
         
         res=[];
         for ii=1:cycles,
            BytesAvailable=0;
            timeoutreached=0;
            while BytesAvailable<packetsinthissequence*8/cycles & timeoutreached==0,
                 pause(.001);
                 BytesAvailable = TCPHandle.BytesAvailable;
                 if toc(timeouttic)>2
                     timeoutreached=1;
                     disp('Timeout reached before data received!');
                     break;
                 end;
            end;
            
            if timeoutreached,
               break;
            end;
            %toc(timeouttic) % 17 ms for 512 addr
            %toc(timeouttic) % 622 ms for 2^16 addr
            
            if BytesAvailable>=packetsinthissequence*8/cycles, % all bytes in buffer
                 A = fread(TCPHandle, packetsinthissequence*4/cycles, 'uint16'); % 2 Bytes per Sample
                 res=[res reshape(A, 4,packetsinthissequence/cycles)];
                 ok=1;
             else
                 ok=0;
            end;
         end;
      end





         
         

   end;

end